/*
 *	subtitle editor
 *
 *	http://kitone.free.fr/subtitleeditor/
 *
 *	Copyright @ 2005-2006, kitone
 *
 *	Contact: kitone at free dot fr
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public
 *	License as published by the Free Software Foundation; either
 *	version 2 of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the GNU
 *	General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public
 *	License along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA	02111-1307	USA
 *
 *	See gpl.txt for more information regarding the GNU General Public License.
 *
 *
 *	\file
 *	\brief 
 *	\author kitone (kitone at free dot fr)
 */

#include "SubtitleTime.h"
#include <stdio.h>
#include <stdlib.h>
#include <glib.h>
#include <iostream>

/**
 *
**/
SubtitleTime::SubtitleTime()
:hours(0), mins(0), secs(0), msecs(0), totalmsecs(0)
{

}

SubtitleTime::SubtitleTime(const std::string &str)
{
	if(sscanf(str.c_str(), "%d:%d:%d,%d", &hours, &mins, &secs, &msecs) != 4)
	{
		hours = mins = secs = msecs = totalmsecs = 0;
	}
	else
		totalmsecs = getMSecs(*this);
}

SubtitleTime::SubtitleTime(const Glib::ustring &str)
{
	if(sscanf(str.c_str(), "%d:%d:%d,%d", &hours, &mins, &secs, &msecs) != 4)
	{
		hours = mins = secs = msecs = totalmsecs = 0;
	}
	else
		totalmsecs = getMSecs(*this);
}

/**
 *
**/
SubtitleTime::SubtitleTime(const int &h, const int &m, const int &s, const int &ms)
{
	set(h,m,s,ms);
}

/**
 *
**/
void SubtitleTime::set(const int &h, const int &m, const int &s, const int &ms)
{
	hours = h;
	mins	= m;
	secs	= s;
	msecs = ms;

	totalmsecs = getMSecs(*this);
}

void SubtitleTime::initTotalMSecs()
{
	totalmsecs = getMSecs(*this);
}


void SubtitleTime::move(long msecs)
{
	totalmsecs+=msecs;
	*this = getTime2MSecs(totalmsecs);
}



SubtitleTime SubtitleTime::operator-(const SubtitleTime &b) const
{
	long total = totalmsecs - b.totalmsecs;
	return getTime2MSecs(total);
}

SubtitleTime SubtitleTime::operator+(const SubtitleTime &b) const
{
	long total = totalmsecs + b.totalmsecs;
	return getTime2MSecs(total);
}

double SubtitleTime::operator/(const SubtitleTime &b) const
{
	//std::cout << "/ " << totalmsecs << " " << b.totalmsecs << " : " << totalmsecs / b.totalmsecs << std::endl;
	double total = totalmsecs / b.totalmsecs;
	//std::cout << "/ = " << total << std::endl;
	return total;
}

SubtitleTime SubtitleTime::operator*(const double &mult) const
{
	double total = (double)(totalmsecs * mult);
	return getTime2MSecs((long int)total);
}


bool SubtitleTime::operator>(const SubtitleTime &time)
{
	return totalmsecs > time.totalmsecs;
}

bool SubtitleTime::operator>=(const SubtitleTime &time)
{
	return totalmsecs >= time.totalmsecs;
}

bool SubtitleTime::operator<(const SubtitleTime &time)
{
	return totalmsecs < time.totalmsecs;
}

bool SubtitleTime::operator<=(const SubtitleTime &time)
{
	return totalmsecs <= time.totalmsecs;
}


std::string SubtitleTime::str()
{
	return getTime2String(*this);
}

/*
 *	return hours:mins:secs,msecs
 */
std::string getTime2String(const SubtitleTime &time)
{
	gchar *tmp = g_strdup_printf("%.2i:%.2i:%.2i,%.3i", 
								time.hours, time.mins, time.secs, time.msecs);
	std::string str(tmp);
	g_free(tmp);
	
	return str;
}

/*
 *
 */
long getMSecs(const SubtitleTime &time)
{
	return (( time.hours * 60 * 60 +
						time.mins  * 60 +
						time.secs) * 1000) +
						time.msecs;
}

/*
 *
 */
SubtitleTime getTime2MSecs(const long &total)
{
	ldiv_t divres;
	
	divres = ldiv(total, 1000);
	long hms = divres.quot;
	int msecs = divres.rem;
	
	divres = ldiv(hms, 60);
	long hm = divres.quot;
	int secs = divres.rem;

	divres = ldiv(hm, 60);
	long h = divres.quot;
	int mins = divres.rem;
	
	int hours = (int)h;
	
	return SubtitleTime(hours, mins, secs, msecs);
}
